"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDirectory = void 0;
const AbstractEntry_1 = require("./AbstractEntry");
const AbstractFile_1 = require("./AbstractFile");
const core_1 = require("./core");
const errors_1 = require("./errors");
const util_1 = require("./util");
class AbstractDirectory extends AbstractEntry_1.AbstractEntry {
    constructor(fs, path) {
        super(fs, path);
    }
    async _copy(toDir, options, errors) {
        if (this.stats) {
            await this.stat(options);
        }
        const fromItems = await this._list(options, errors);
        if (!fromItems) {
            return false;
        }
        const to = toDir;
        try {
            await to._validate(options);
            if (options.onExists === core_1.ExistsAction.Skip) {
                return true;
            }
            if (options.onExists === core_1.ExistsAction.Error) {
                await this.fs._handleError({
                    ...errors_1.InvalidModificationError,
                    path: this.path,
                    from: this.path,
                    to: to.path,
                }, errors);
                return false;
            }
        }
        catch (e) {
            if ((0, errors_1.isFileSystemError)(e)) {
                if (e.name !== errors_1.NotFoundError.name) {
                    throw e;
                }
            }
            else {
                throw e;
            }
        }
        let result = await to.mkcol(options, errors);
        if (!result) {
            return false;
        }
        if (!options.recursive) {
            return true;
        }
        const fs = this.fs;
        for (const fromItem of fromItems) {
            const fromPath = fromItem.path;
            const fromEntry = await fs.getEntry(fromPath, {
                type: fromItem.type,
                ignoreHook: options.ignoreHook,
            }, errors);
            const name = (0, util_1.getName)(fromPath);
            const toPath = (0, util_1.joinPaths)(to.path, name);
            let copyResult;
            if (fromEntry instanceof AbstractFile_1.AbstractFile) {
                const toEntry = fs.getFile(toPath);
                copyResult = await fromEntry._copy(toEntry, { ...options, type: core_1.EntryType.File }, errors);
            }
            else if (fromEntry instanceof AbstractDirectory) {
                const toEntry = fs.getDirectory(toPath);
                copyResult = await fromEntry._copy(toEntry, { ...options, type: core_1.EntryType.Directory }, errors);
            }
            else {
                continue;
            }
            result = copyResult && result;
        }
        return result;
    }
    async _deleteExisting(options, errors) {
        let result = true;
        if (options.recursive) {
            const children = await this._list(options, errors);
            if (!children) {
                return false;
            }
            for (const child of children) {
                const childEntry = await this.fs.getEntry(child.path, {
                    type: child.type,
                    ignoreHook: options.ignoreHook,
                }, errors);
                if (!childEntry) {
                    result = false;
                    continue;
                }
                result = (await childEntry.delete(options, errors)) && result;
            }
        }
        if (result) {
            await this._doDelete();
        }
        return result;
    }
    async _validate(options) {
        if (this.stats) {
            if (this.stats.size != null) {
                throw this._createTypeMismatchError({
                    message: `"${this.path}" is not a directory`,
                });
            }
        }
        else {
            await this.head(options);
        }
    }
    async dir(options, errors) {
        return await this.list(options, errors);
    }
    async head(options, errors) {
        if (!this.fs.supportDirectory()) {
            this.stats = {};
            return this.stats;
        }
        options = { ...options, type: core_1.EntryType.Directory };
        this.stats = await this.fs.head(this.path, options, errors);
        return this.stats;
    }
    async list(options, errors) {
        options = { ...options };
        const list = await this._list(options, errors);
        if (!list) {
            return null;
        }
        return list.map((item) => item.path);
    }
    async ls(options, errors) {
        return await this.list(options, errors);
    }
    async mkcol(options, errors) {
        options = { ...this.fs.defaultMkcolOptions, ...options };
        try {
            const result = await this.__mkcol(options);
            await this._afterMkcol(options, result);
            return result;
        }
        catch (e) {
            const opts = options;
            await this._handleNoModificationAllowedError({ e }, errors, async (error) => {
                await this._afterMkcol(opts, false, error);
            });
            return false;
        }
        finally {
            this.stats = undefined;
        }
    }
    mkdir = (options) => this.mkcol(options);
    async readdir(options, errors) {
        return await this.list(options, errors);
    }
    async __list(options) {
        await this._validate(options);
        const list = await this._doList();
        for (const item of list) {
            if (item.path.endsWith("/")) {
                if (!item.type) {
                    item.type = core_1.EntryType.Directory;
                }
            }
            item.path = (0, util_1.normalizePath)(item.path);
        }
        return list;
    }
    async __mkcol(options) {
        if (!this.fs.supportDirectory()) {
            return true;
        }
        if (this.path !== "/") {
            const parent = this.getParent();
            try {
                await parent.head({
                    type: core_1.EntryType.Directory,
                    ignoreHook: options?.ignoreHook,
                });
            }
            catch (e) {
                if ((0, errors_1.isFileSystemError)(e) && e.name === errors_1.NotFoundError.name) {
                    if (options.onNoParent === core_1.NoParentAction.Error) {
                        throw e;
                    }
                    if (parent.path !== "/") {
                        await parent.mkcol(options);
                    }
                }
            }
        }
        try {
            await this._validate(options);
            if (options.onExists === core_1.ExistsAction.Error) {
                throw this._createError({
                    ...errors_1.PathExistError,
                    message: `"${this.path}" has already existed`,
                });
            }
            return true;
        }
        catch (e) {
            if ((0, errors_1.isFileSystemError)(e)) {
                if (e.name !== errors_1.NotFoundError.name) {
                    throw e;
                }
            }
            else {
                throw e;
            }
        }
        await this._doMkcol();
        return true;
    }
    async _afterList(options, list, error) {
        const fs = this.fs;
        const afterList = fs.options.hook?.afterList;
        if (afterList && !options.ignoreHook) {
            await afterList(fs.repository, this.path, options, list, error);
        }
    }
    async _afterMkcol(options, result, error) {
        const fs = this.fs;
        const afterMkcol = fs.options.hook?.afterMkcol;
        if (afterMkcol && !options.ignoreHook) {
            await afterMkcol(fs.repository, this.path, options, result, error);
        }
    }
    async _beforeList(options) {
        const fs = this.fs;
        const beforeList = fs.options.hook?.beforeList;
        if (beforeList && !options.ignoreHook) {
            return await beforeList(fs.repository, this.path, options);
        }
        return null;
    }
    async _beforeMkcol(options) {
        const fs = this.fs;
        const beforeMkcol = fs.options.hook?.beforeMkcol;
        if (beforeMkcol && !options.ignoreHook) {
            return await beforeMkcol(fs.repository, this.path, options);
        }
        return null;
    }
    async _list(options, errors) {
        try {
            let list = await this._beforeList(options);
            if (!list) {
                list = await this.__list(options);
            }
            await this._afterList(options, list);
            return list;
        }
        catch (e) {
            await this._handleNotReadableError({ e }, errors, async (error) => {
                await this._afterList(options, null, error);
            });
            return null;
        }
    }
}
exports.AbstractDirectory = AbstractDirectory;
//# sourceMappingURL=AbstractDirectory.js.map