"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFile = void 0;
const sha256_uint8array_1 = require("sha256-uint8array");
const univ_conv_1 = require("univ-conv");
const AbstractEntry_1 = require("./AbstractEntry");
const core_1 = require("./core");
const errors_1 = require("./errors");
const mods_1 = require("./mods");
const util_1 = require("./util");
class AbstractFile extends AbstractEntry_1.AbstractEntry {
    constructor(fs, path) {
        super(fs, path);
    }
    async _copy(toFile, options, errors) {
        const to = toFile;
        try {
            await this._validate(options);
            if (options.onExists === core_1.ExistsAction.Skip) {
                return true;
            }
            if (options.onExists === core_1.ExistsAction.Error) {
                await this.fs._handleError({
                    ...errors_1.InvalidModificationError,
                    path: this.path,
                    from: this.path,
                    to: to.path,
                }, errors);
                return false;
            }
        }
        catch (e) {
            if ((0, errors_1.isFileSystemError)(e)) {
                if (e.name !== errors_1.NotFoundError.name) {
                    throw e;
                }
            }
            else {
                throw e;
            }
        }
        const data = await this._read(options, errors);
        if (data == null) {
            return false;
        }
        return to._write(data, options, errors);
    }
    async _deleteExisting() {
        await this._doDelete();
        return true;
    }
    async _validate(options) {
        if (this.stats) {
            if (this.stats.size == null) {
                throw this._createTypeMismatchError({
                    message: `"${this.path}" is not a file`,
                });
            }
        }
        else {
            await this.head(options);
        }
    }
    async _write(data, options, errors) {
        options = { ...this.fs.defaultWriteOptions, ...options };
        if (options.append && options.start != null) {
            options.append = false;
            console.warn("Set options.append to false because options.start is not null.");
        }
        try {
            try {
                await this._validate(options);
                if (options.onExists === core_1.ExistsAction.Skip) {
                    return true;
                }
                if (options.onExists === core_1.ExistsAction.Error) {
                    throw this._createError(errors_1.PathExistError);
                }
            }
            catch (e) {
                if ((0, errors_1.isFileSystemError)(e)) {
                    if (e.name === errors_1.NotFoundError.name) {
                        if (options.onNotExist === core_1.NotExistAction.Error) {
                            throw e;
                        }
                    }
                    else {
                        throw e;
                    }
                }
                else {
                    throw e;
                }
            }
            const result = await this.$write(data, options);
            if (this.stats) {
                await this._afterPut(options, result);
            }
            else {
                await this._afterPost(options, result);
            }
            return result;
        }
        catch (e) {
            const opts = options;
            await this._handleNoModificationAllowedError({ e }, errors, async (error) => {
                if (this.stats) {
                    await this._afterPut(opts, false, error);
                }
                else {
                    await this._afterPost(opts, false, error);
                }
            });
            return false;
        }
        finally {
            this.stats = undefined;
        }
    }
    async hash(options, errors) {
        options = { ...options };
        const data = await this._read(options, errors);
        if (data == null) {
            return null;
        }
        const hash = (0, sha256_uint8array_1.createHash)();
        if ((0, univ_conv_1.readableConverter)().typeEquals(data)) {
            await (0, univ_conv_1.handleReadable)(data, async (chunk) => {
                const buffer = await (0, univ_conv_1.bufferConverter)().convert(chunk, {
                    bufferSize: options?.bufferSize,
                });
                hash.update(buffer);
                return true;
            });
        }
        else if ((0, univ_conv_1.readableStreamConverter)().typeEquals(data)) {
            await (0, univ_conv_1.handleReadableStream)(data, async (chunk) => {
                const u8 = await (0, univ_conv_1.uint8ArrayConverter)().convert(chunk, {
                    bufferSize: options?.bufferSize,
                });
                hash.update(u8);
                return true;
            });
        }
        else {
            const u8 = await (0, univ_conv_1.uint8ArrayConverter)().convert(data);
            hash.update(u8);
        }
        return (0, util_1.toHex)(hash.digest());
    }
    async head(options, errors) {
        options = { ...options, type: core_1.EntryType.File };
        try {
            this.stats = await this.fs.head(this.path, options, errors);
        }
        catch (e) {
            this.stats = null;
            throw e;
        }
        return this.stats;
    }
    async read(type, options, errors) {
        options = { ...this.fs.defaultReadOptions, ...options };
        const data = await this._read(options, errors);
        if (data === null) {
            return null;
        }
        if (type == null) {
            return data;
        }
        const converter = this._getConverter();
        return await converter.convert(data, type, options);
    }
    async write(data, options, errors) {
        return await this._write(data, options, errors);
    }
    async $write(data, options) {
        const length = options.length;
        if (length === 0) {
            return true;
        }
        if (this.stats) {
            const result = await this._beforePut(data, options);
            if (result != null) {
                return result;
            }
        }
        else {
            const result = await this._beforePost(data, options);
            if (result != null) {
                return result;
            }
        }
        const start = options.start;
        const rc = (0, univ_conv_1.readableConverter)();
        const rsc = (0, univ_conv_1.readableStreamConverter)();
        if (!this.supportAppend() && options.append) {
            options.append = false;
            const head = await this._read({ bufferSize: options.bufferSize });
            const converter = this._getConverter();
            if (rc.typeEquals(head) || rc.typeEquals(data)) {
                data = await converter.merge([head, data], "readable");
            }
            else if (rsc.typeEquals(head) || rsc.typeEquals(data)) {
                data = await converter.merge([head, data], "readablestream");
            }
            else if (univ_conv_1.isBrowser) {
                data = await converter.merge([head, data], "blob");
            }
            else if (univ_conv_1.isNode) {
                data = await converter.merge([head, data], "buffer");
            }
            else {
                data = await converter.merge([head, data], "uint8array");
            }
        }
        else if (!this.supportRangeWrite() &&
            (typeof start === "number" || typeof length === "number")) {
            delete options.start;
            delete options.length;
            const src = await this._read({ bufferSize: options.bufferSize });
            if (rc.typeEquals(src)) {
                data = new mods_1.ModifiedReadable(src, { data, start, length });
            }
            else if (rsc.typeEquals(src)) {
                data = (0, mods_1.createModifiedReadableStream)(src, { data, start, length });
            }
            else {
                data = await (0, mods_1.modify)(src, { data, start, length });
            }
        }
        await this._doWrite(data, this.stats, options);
        return true;
    }
    async __read(options) {
        if (options.length === 0) {
            return univ_conv_1.EMPTY_UINT8_ARRAY;
        }
        await this._validate(options);
        const stats = this.stats;
        if (stats.size === 0) {
            return univ_conv_1.EMPTY_UINT8_ARRAY;
        }
        let data = await this._doRead(stats, options);
        if (!this.supportRangeRead() &&
            (typeof options?.start === "number" ||
                typeof options?.length === "number")) {
            data = await univ_conv_1.DEFAULT_CONVERTER.slice(data, options);
        }
        return data;
    }
    async _afterGet(options, data, error) {
        const fs = this.fs;
        const afterGet = fs.options.hook?.afterGet;
        if (afterGet && !options.ignoreHook) {
            await afterGet(fs.repository, this.path, options, data, error);
        }
    }
    async _afterPost(options, result, error) {
        const fs = this.fs;
        const afterPost = fs.options.hook?.afterPost;
        if (afterPost && !options.ignoreHook) {
            await afterPost(fs.repository, this.path, options, result, error);
        }
    }
    async _afterPut(options, result, error) {
        const fs = this.fs;
        const afterPut = fs.options.hook?.afterPut;
        if (afterPut && !options.ignoreHook) {
            await afterPut(fs.repository, this.path, options, result, error);
        }
    }
    async _beforeGet(options) {
        const fs = this.fs;
        const beforeGet = fs.options.hook?.beforeGet;
        if (beforeGet && !options.ignoreHook) {
            return await beforeGet(fs.repository, this.path, options);
        }
        return null;
    }
    async _beforePost(data, options) {
        const fs = this.fs;
        const beforePost = fs.options.hook?.beforePost;
        if (beforePost && !options.ignoreHook) {
            await beforePost(fs.repository, this.path, data, options);
        }
    }
    async _beforePut(data, options) {
        const fs = this.fs;
        const beforePut = fs.options.hook?.beforePut;
        if (beforePut && !options.ignoreHook) {
            await beforePut(fs.repository, this.path, data, options);
        }
    }
    _getConverter() {
        return univ_conv_1.DEFAULT_CONVERTER;
    }
    async _read(options, errors) {
        try {
            let data = await this._beforeGet(options);
            if (data) {
                return data;
            }
            data = await this.__read(options);
            await this._afterGet(options, data);
            return data;
        }
        catch (e) {
            const opts = options;
            await this._handleNotReadableError({ e }, errors, async (error) => {
                await this._afterGet(opts, null, error);
            });
            return null;
        }
    }
}
exports.AbstractFile = AbstractFile;
//# sourceMappingURL=AbstractFile.js.map