"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testAll = void 0;
const univ_conv_1 = require("univ-conv");
const core_1 = require("../core");
const errors_1 = require("../errors");
const c = univ_conv_1.DEFAULT_CONVERTER;
const testAll = (fs, options) => {
    it("setup", async () => {
        if (options?.setup) {
            await options.setup();
        }
    });
    it("rootdir", async () => {
        const dir = fs.getDirectory("/");
        const paths = await dir.readdir();
        expect(paths.length).toBe(0);
    });
    it("add empty file", async () => {
        const file = fs.getFile("/empty.txt");
        try {
            await file.stat();
            throw new Error("Found file: " + file.path);
        }
        catch (e) {
            expect(e.name).toBe(errors_1.NotFoundError.name);
        }
        const buffer = await c.toArrayBuffer("");
        await file.write(buffer);
        const stats = await file.stat();
        expect(stats.size).toBe(0);
    });
    it("add text file", async () => {
        const file = fs.getFile("/test.txt");
        try {
            await file.stat();
            throw new Error("Found file: " + file.path);
        }
        catch (e) {
            expect(e.name).toBe(errors_1.NotFoundError.name);
        }
        const buffer = await c.toArrayBuffer("test");
        await file.write(buffer);
        const stats = await file.stat();
        expect(stats.size).toBe(4);
    });
    it("read text file", async () => {
        const file = fs.getFile("/test.txt");
        const buffer = await file.read("uint8array");
        expect(buffer.byteLength).toBe(4);
        const text = await c.toText(buffer);
        expect(text).toBe("test");
    });
    it("continuous read and write", async () => {
        const file = fs.getFile("/otani.txt");
        await file.write("大谷翔平");
        let text = await file.read("text");
        expect(text).toBe("大谷翔平");
        await file.write("ホームラン", {
            append: true,
            onExists: core_1.ExistsAction.Overwrite,
        });
        text = await file.read("text");
        expect(text).toBe("大谷翔平ホームラン");
    });
    it("listdir test", async () => {
        const dir = fs.getDirectory("/");
        let dirs = await dir.readdir();
        expect(0 <= dirs.indexOf("/empty.txt")).toBe(true);
        expect(0 <= dirs.indexOf("/test.txt")).toBe(true);
        expect(0 <= dirs.indexOf("/otani.txt")).toBe(true);
    });
    it("mkdir test", async () => {
        const folder = fs.getDirectory("/folder");
        try {
            const stats = await folder.stat();
            if (stats.size != null) {
                throw new Error("Found file: " + folder.path);
            }
        }
        catch (e) {
            expect(e.name).toBe(errors_1.NotFoundError.name);
        }
        await folder.mkdir();
        try {
            const stats = await folder.stat();
            expect(stats).not.toBe(null);
            if (stats.size != null) {
                throw new Error("File has created: " + folder.path);
            }
        }
        catch (e) {
            expect(e.name).toBe(errors_1.NotFoundError.name);
        }
    });
    it("create file in dir", async () => {
        let file = fs.getFile("/folder/sample.txt");
        file = file;
        try {
            await file.stat();
            throw new Error("Found file: " + file.path);
        }
        catch (e) {
            expect(e.name).toBe(errors_1.NotFoundError.name);
        }
        const before = Math.floor(Date.now() / 1000);
        await file.write("Sample");
        const after = Math.floor(Date.now() + 1 / 1000);
        const stats = await file.stat();
        const modified = Math.floor((stats.modified ?? 0) / 1000);
        expect(modified).toBeGreaterThanOrEqual(before);
        expect(modified).toBeLessThan(after);
        const text = await file.read("text");
        expect(text).toBe("Sample");
        const dir = fs.getDirectory("/folder/");
        const list = await dir.list();
        expect(0 <= list.indexOf("/folder/sample.txt")).toBe(true);
    });
    it("copy directory", async () => {
        const from = fs.getDirectory("/folder");
        const to = fs.getDirectory("/folder2");
        await from.copy(to, {
            onExists: core_1.ExistsAction.Overwrite,
            onNoParent: core_1.NoParentAction.Error,
            recursive: true,
        });
        const stats = await to.stat();
        expect(stats.size).toBeUndefined();
        if (fs.supportDirectory()) {
            const root = fs.getDirectory("/");
            const list = await root.ls();
            expect(0 <= list.indexOf("/folder2")).toBe(true);
        }
        const toList = await to.ls();
        expect(0 <= toList.indexOf("/folder2/sample.txt")).toBe(true);
    });
    it("move file", async () => {
        await fs.move("/folder2/sample.txt", "/folder2/sample2.txt", {
            onExists: core_1.ExistsAction.Overwrite,
            onNoParent: core_1.NoParentAction.Error,
        });
        const list = await fs.list("/folder2");
        expect(list.indexOf("/folder2/sample.txt") < 0).toBe(true);
        expect(0 <= list.indexOf("/folder2/sample2.txt")).toBe(true);
    });
    it("move directory", async () => {
        await fs.move("/folder2", "/folder3", {
            onExists: core_1.ExistsAction.Overwrite,
            onNoParent: core_1.NoParentAction.Error,
        });
        if (fs.supportDirectory()) {
            const root = fs.getDirectory("/");
            const list = await root.ls();
            expect(list.indexOf("/folder2") < 0).toBe(true);
            expect(0 <= list.indexOf("/folder3")).toBe(true);
        }
        const folder3 = fs.getDirectory("/folder3");
        const folder3List = await folder3.ls();
        expect(0 <= folder3List.indexOf("/folder3/sample2.txt")).toBe(true);
    });
    it("teardown", async () => {
        if (options?.teardown) {
            await options.teardown();
        }
    });
};
exports.testAll = testAll;
//# sourceMappingURL=basic.js.map